#ifndef CHORD
#define CHORD

#include <stdint.h>
#include <netinet/in.h>

#ifdef __cplusplus
extern "C" {
#endif

struct chord_opts {
	unsigned int join : 1;
	const char *local_node;
	const char *local_service;
	const char *node;
	const char *service;
	int hash_size;
	int num_successors;
	int (*handler)(int socketfd, void *arg);
	void *arg;
};

struct chord_node {
	unsigned char *hash;
	uint32_t address;
	in_port_t port;
};

struct chord_node_list {
	int num_nodes;
	struct chord_node *nodes;
};

struct chord_network;

int chord_successor_ext(const struct chord_node *node, size_t hash_size,
		const unsigned char *hash, struct chord_node *successor);

int chord_init_node(const char *node, const char *service, struct chord_node *chord_node);
int chord_oob(struct chord_node *node);
int chord_successor(struct chord_network *network, unsigned char *hash,
		struct chord_node *successor);

int chord_start(const struct chord_opts *options,
		struct chord_network **network);

void chord_stop(struct chord_network *network);
int chord_get_successors(struct chord_network *network,
		struct chord_node_list *list);

void chord_node_list_destroy(struct chord_node_list *list);

#define for_each_node(node, list) \
	for (node = list->nodes; node != list->nodes + list->num_nodes; node++)

#define push_chord_node_list_destroy(list) \
	pthread_cleanup_push((void (*)(void *)) chord_node_list_destroy, list)

#ifdef __cplusplus
}
#endif

#endif
