#include <chord.h>

#include <error.h>
#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>

struct chord_network *network;

static void parse_options(int argc, char *argv[], struct chord_opts *options)
{
	const struct option opts[] = {
		{
			.name = "join",
			.has_arg = no_argument,
			.flag = NULL,
			.val = 'j'
		},
		{
			.name = 0,
			.has_arg = 0,
			.flag = 0,
			.val = 0
		}
	};
	const char short_opts[] = "j";
	int opt;

	while ((opt = getopt_long(argc, argv, short_opts, opts, NULL)) != -1) {
		switch (opt) {
		case 'j':
			options->join = 1;
			break;
		default:
			exit(EXIT_FAILURE);
		}
	}

	if (options->join) {
		options->node = *(argv + optind++);
		options->service = *(argv + optind++);
	}

	options->local_node = *(argv + optind++);
	options->local_service = *(argv + optind++);
	if (argc - optind)
		error(EXIT_FAILURE, 0, "incorrect usage");
}

static void cleanup_handler(int signum)
{
	chord_stop(network);
	exit(EXIT_SUCCESS);
}

int main(int argc, char *argv[])
{
	struct chord_opts options = {
		.join = 0,
		.local_node = NULL,
		.local_service = NULL,
		.node = NULL,
		.service = NULL,
		.hash_size = 0,
		.num_successors = 0
	};
	struct sigaction cleanup = {
		.sa_handler = cleanup_handler,
		.sa_flags = 0
	};

	parse_options(argc, argv, &options);

	if (chord_start(&options, &network) == -1)
		return -1;

	sigfillset(&cleanup.sa_mask);
	sigaction(SIGINT, &cleanup, NULL);

	while (1)
		sleep(1);

	chord_stop(network);

	exit(EXIT_SUCCESS);
}
