#include "socket.h"
#include "hash.h"

#include <openssl/evp.h>

#include <cstring>

p2pvdn::hash::hash(const std::string &path, unsigned size)
	: key(size)
{
	std::string filename = path.substr(path.rfind('/') + 1, std::string::npos);

	EVP_MD_CTX mdctx;
	unsigned char hash[EVP_MAX_MD_SIZE];
	unsigned hash_len;
	EVP_MD_CTX_init(&mdctx);
	EVP_DigestInit_ex(&mdctx, EVP_sha1(), NULL);
	EVP_DigestUpdate(&mdctx, filename.c_str(), filename.size());
	EVP_DigestFinal_ex(&mdctx, hash, &hash_len);
	EVP_MD_CTX_cleanup(&mdctx);

	std::copy(hash, hash + std::min<unsigned>(key.size(), hash_len), key.begin());
}

bool p2pvdn::hash::operator<(const hash &hash) const throw()
{
	unsigned min_size = std::min(key.size(), hash.key.size());
	int retval = std::memcmp(&key[0], &hash.key[0], min_size);

	if (retval)
		return retval < 0 ? true : false;

	return key.size() < hash.key.size() ? true : false;
}
