#ifndef PEER_H
#define PEER_H

#include <iostream>

#include <stdint.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

namespace p2pvdn {
	class node {
		uint32_t address;
		in_port_t port;

		public:
		node()
		{
		}

		node(const socket &socket)
		{
			socket.recvall(&address, sizeof(address));
			socket.recvall(&port, sizeof(port));
		}

		node(uint32_t address, in_port_t port)
			: address(address), port(port)
		{
		}

		uint32_t get_address() const
		{
			return address;
		}

		in_port_t get_port() const
		{
			return port;
		}

		bool operator<(const node &node) const
		{
			if (address < node.address)
				return true;

			if (address == node.address && port < node.port)
				return true;

			return false;
		}

		friend const socket &operator<<(const socket &socket, const node &node);
		friend const socket &operator>>(const socket &socket, node &node);
	};

	inline const socket &operator<<(const socket &socket, const node &node)
	{
		socket.sendall(&node.address, sizeof(node.address));
		socket.sendall(&node.port, sizeof(node.port));

		return socket;
	}

	inline const socket &operator>>(const socket &socket, node &node)
	{
		socket.recvall(&node.address, sizeof(node.address));
		socket.recvall(&node.port, sizeof(node.port));

		return socket;
	}

	inline std::ostream &operator<<(std::ostream &stream, const node &node)
	{
		stream << inet_ntoa({node.get_address()}) << ':' << ntohs(node.get_port());

		return stream;
	}
}

#endif
