#! /bin/bash -e

VDND=../vdnd/vdnd
DISPERSE=../disperse/disperse
GET_FILE=../get_file/get_file

if [ -z "$port" ]; then
	port=50002
fi

if [ -z "$packet_size" ]; then
	packet_size=2048
fi

if [ -z "$file_size" ]; then
	file_size=$((2**14))
fi

if [ -z "$num_slices" ]; then
	num_slices=20
fi

usage() {
	:
}

error() {
	local message="$1"

	if [ -n "$message" ]; then
		echo "$message"
	fi
	exit 1
}

node_address() {
	declare -i node_num="$1"+1 octet1 octet2 octet3

	((octet1=$node_num%256))
	((node_num/=256))
	((octet2=$node_num%256))
	((node_num/=256))
	((octet3=$node_num%256))
	echo "127.$octet3.$octet2.$octet1"
}

disperse_file() {
	declare file="$1"
	declare -i node_num="$2"
	declare -a disperse_args=( "$file" -a $(node_address $node_num) )

	if [ -n "$port" ]; then
		disperse_args=( "${disperse_args[@]}" -p $port )
	fi
	if [ -n "$packet_size" ]; then
		disperse_args=( "${disperse_args[@]}" -s $packet_size )
	fi
	if [ -n "$port" ]; then
		disperse_args=( "${disperse_args[@]}" -n $num_slices )
	fi

	"$DISPERSE" "${disperse_args[@]}"
}

declare -i num_nodes="$1" node_num
declare -a pids

if [ $num_nodes -lt 1 ]; then
	usage
	error "must use at least one node"
fi

for node_num in $(seq 0 $((num_nodes-1))); do
	declare -a vdnd_args=( -a $(node_address $node_num) )
	if [ -n "$port" ]; then
		vdnd_args=( "${vdnd_args[@]}" -p "$port" )
	fi
	if [ $node_num -ne 0 ]; then
		vdnd_args=( "${vdnd_args[@]}" 
				-j $(node_address $(($node_num-1))) )
		if [ -n "$port" ]; then
			vdnd_args=( "${vdnd_args[@]}" -P "$port" )
		fi
	fi
	$VDND "${vdnd_args[@]}" &
	pids[$node_num]=$!
	sleep 1
done

file=$(mktemp)
dd if=/dev/urandom of="$file" count=$file_size &> /dev/null
disperse_file "$file" 0
if [ -n "$port" ]; then
	/usr/bin/time -o $file -f "%e" "$GET_FILE" "$file" /dev/null -p "$port" &>/dev/null
else
	/usr/bin/time -o $file -f "%e" "$GET_FILE" "$file" /dev/null &>/dev/null
fi
time=$(cat $file)
rm -f "$file"

for pid in ${pids[@]}; do
	kill $pid
done

echo time: $time
echo -n throughput:
bc <<<"$file_size * 512 / $time"
