#include "peer_data.h"

#include <thread>
#include <iostream>

const p2pvdn::socket &vdnd::operator>>(const p2pvdn::socket &socket, peer_data &peer_data)
{
	std::vector<std::pair<p2pvdn::node, unsigned>> peers;
	socket >> peers;
	std::lock_guard<boost::shared_mutex> lock{peer_data.mutex};
	for (auto peer = peers.begin(); peer != peers.end(); peer++)
		peer_data.peers[peer->first] += peer->second;

	return socket;
}

const p2pvdn::socket &vdnd::operator<<(const p2pvdn::socket &socket, const peer_data &peer_data)
{
	boost::shared_lock<boost::shared_mutex> lock{peer_data.mutex};
	std::vector<std::pair<p2pvdn::node, unsigned>> peers(peer_data.peers.begin(), peer_data.peers.end());
	lock.unlock();
	socket << peers;

	return socket;
}

std::ostream &vdnd::operator<<(std::ostream &stream, const peer_data &peer_data)
{
	boost::shared_lock<boost::shared_mutex> lock {peer_data.mutex};
	for (auto peer = peer_data.peers.begin(); peer != peer_data.peers.end(); peer++)
		stream << "node: " << peer->first << " slices: " << peer->second << std::endl;

	return stream;
}
