#ifndef SHARED_MAP_H
#define SHARED_MAP_H

#include <boost/thread/shared_mutex.hpp>

#include <map>
#include <memory>
#include <thread>
#include <algorithm>

namespace vdnd {
	template<class key_type, class data_type> class shared_map {
		mutable boost::shared_mutex mutex;
		std::map<key_type, std::shared_ptr<data_type>> map;

		public:
		std::shared_ptr<data_type> get(key_type &&) const;
		template<class... data_args> std::shared_ptr<data_type> add(key_type &&, data_args&&...);
	};
}

template<class key_type, class data_type>
std::shared_ptr<data_type> vdnd::shared_map<key_type, data_type>::get(key_type &&key) const
{
	boost::shared_lock<boost::shared_mutex> read_lock {mutex};
	auto data = map.find(std::forward<key_type>(key));
	if (data == map.end())
		throw std::range_error("key not found");

	return data->second;
}

template<class key_type, class data_type> template<class... arg_types>
std::shared_ptr<data_type> vdnd::shared_map<key_type, data_type>::add(key_type &&key, arg_types &&... args)
{
	//TODO: upgrade_lock (once boost fixes C++0x compatibility)
	std::lock_guard<boost::shared_mutex> lock {mutex};
	auto &data = map[std::forward<key_type>(key)];
	if (!data.get())
		data.reset(new data_type(std::forward<arg_types>(args)...));

	return data;
}

#endif
